uniform mediump vec4 u_screenSize;
uniform highp float u_time;
uniform mediump sampler2D u_texture; // Diffuse
//uniform mediump sampler2D u_texture1; // Normal
uniform mediump sampler2D u_texture1; // Mask
//uniform mediump sampler2D u_texture3; // Highlight
varying mediump vec2 v_texCoord;
varying mediump vec4 v_color;

varying mediump vec3 v_normal;
varying mediump vec3 v_tangent;
varying mediump vec3 v_binormal;

varying mediump vec3 v_camDir;
varying mediump vec3 v_camPos;
varying mediump vec3 v_lightDir;

uniform mediump vec4 u_color4;
uniform mediump vec4 u_materialAmbient;
uniform mediump vec4 u_materialDiffuse;
uniform mediump vec4 u_materialSpecular;
uniform mediump vec4 u_materialEmissive;
uniform mediump float u_materialShininess;

uniform highp vec4 u_data1; // x=speed, y=offset, z=tile, w=alpha
uniform highp vec4 u_data2; // x=color_adjustment, y=None, z=start_time, w=duration

void main() {
	mediump vec4 color = texture2D(u_texture, v_texCoord * u_data1.z);
	//mediump vec3 normalmap = texture2D(u_texture1, v_texCoord * u_data2.x).xyz * 2.0 - 1.0;
	const mediump vec3 normalmap = vec3(0.0, 0.0, 1.0);
	//mediump vec4 hl = texture2D(u_texture3, gl_FragCoord.xy);

	mediump vec3 L = normalize(v_lightDir);
	//mediump vec3 L = normalize(vec3(0.0, 0.0, -1.0));
	mediump vec3 CD = normalize(v_camDir);
	mediump vec3 N = normalize(v_normal);
	mediump vec3 T = normalize(v_tangent);
	mediump vec3 B = normalize(v_binormal);
	
	// Normal mapping
	mediump mat3 TBN = mat3(T, B, N);
	mediump vec3 Nm = normalize(TBN * normalmap);

	const mediump float intensity = 1.0;
	//mediump float nDotL = clamp(dot(L, Nm), 0.0, 1.0);
	mediump float nDotL = clamp(dot(Nm, L), 0.3, 1.0);
	mediump float diffuse = nDotL * intensity;
	mediump float specular = pow(nDotL, u_materialShininess) * intensity;

	//gl_FragColor = clamp(color + (color * clamp(pow(dot(N, CD), u_data1.z), 0.0, 1.0)), 0.0, 1.0) * u_data2.y;
	//gl_FragColor = clamp(color + (color * clamp(pow(dot(N, CD), u_data1.z), 0.0, 1.0)), 0.0, 1.0) * u_data2.y;
	//gl_FragColor = color + (clamp(pow(dot(N, CD), u_data1.z), 0.0, 1.0)) * 0.5;
	//gl_FragColor = color + 1.0 - clamp(pow(dot(Nm, v_camDir), u_data1.z), 0.0, 1.0);

	gl_FragColor = color;
	gl_FragColor.rgb = gl_FragColor.rgb * (u_materialDiffuse * diffuse + u_materialAmbient).rgb + u_materialSpecular.rgb * specular;
	gl_FragColor.rgb = gl_FragColor.rgb * vec3(u_materialAmbient.a, u_materialAmbient.a, u_materialAmbient.a);
	gl_FragColor = gl_FragColor * u_color4;

	gl_FragColor.rgb *= u_data2.x;
	gl_FragColor.a = u_data1.w;

	//const mediump vec4 iceCreamColor = vec4(1, 0.75, 0.79, 1);
	mediump vec4 transitionColor = vec4(gl_FragColor.rgb, 0.0);
	mediump vec4 mask = texture2D(u_texture1, v_texCoord);
	
	mediump float st = u_time - u_data2.z;
	mediump float t = mod(st * 0.5 * (1.0 / u_data2.w), 1.0);
	//mediump float t = 1.0 - mod(st * 0.2, 1.0); // Reverse
	mediump float R = mask.g;
	t = t - 0.75;
	t = t * 2.0;
	t = abs(t);
	t = t * -1.0;
	t = t + 1.5;
	t = clamp(t, 0.0, 1.0);
	t = 1.0 - t;
	R = R - t;
	R = R * 100.0;
	R = clamp(R, 0.0, 1.0);

	//mediump float spec = (1.0 * clamp(pow(dot(N, CD), u_data1.z), 0.0, 1.0)) * 0.5;
	//gl_FragColor = mix(transitionColor + spec, color, R);
	gl_FragColor = mix(transitionColor, gl_FragColor, R);
}